package de.fhdw.wtf.tooling.builders;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IJavaProject;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.exception.editor.CheckException;
import de.fhdw.wtf.common.exception.editor.CheckExceptionVisitor;
import de.fhdw.wtf.common.exception.editor.GeneralCheckException;
import de.fhdw.wtf.common.exception.editor.MarkableCheckException;
import de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions;
import de.fhdw.wtf.tooling.builders.result.AbstractCommandResult;
import de.fhdw.wtf.tooling.builders.result.NotExecutedResult;
import de.fhdw.wtf.tooling.editor.ConsoleWriter;
import de.fhdw.wtf.tooling.editor.MarkerWriter;

public abstract class IntegrationCommand {
	
	private static final String wtfFileExtension = "wtf";
	private static final String relativeModelFolderPath = "src/main/resources/models";
	
	private final IProject project;
	
	protected IProject getProject() {
		return this.project;
	}
	
	private AbstractCommandResult<Model> result;
	
	protected AbstractCommandResult<Model> getResult() {
		return this.result;
	}
	
	protected void setResult(final AbstractCommandResult<Model> result) {
		this.result = result;
	}
	
	public IntegrationCommand(final IProject project) {
		this.setResult(new NotExecutedResult<Model>());
		this.project = project;
	}
	
	public abstract void execute();
	
	/**
	 * Returns all {@link IResource}s in the model folder of the provided {@link IJavaProject} with the .wtf file
	 * extension.
	 * 
	 * @param javaProject
	 * @return
	 */
	protected Collection<IResource> findModelResources() {
		final Vector<IResource> resources = new Vector<>();
		final IResource modelsFolder = this.project.findMember(relativeModelFolderPath);
		if (modelsFolder instanceof IFolder) {
			try {
				final IResource[] members = ((IFolder) modelsFolder).members();
				for (final IResource member : members) {
					if (wtfFileExtension.equals(member.getFileExtension())) {
						resources.add(member);
					}
				}
			} catch (final CoreException e) {
				// nothing to do
			}
			
		}
		return resources;
	}
	
	protected void reactOnExceptionsSimple(final MultipleCheckExceptions e) {
		final Iterator<CheckException> i = e.iterator();
		while (i.hasNext()) {
			final CheckException current = i.next();
			ConsoleWriter.println(current.getMessage());
		}
	}
	
	/**
	 * Creates a problem marker for each {@link MarkableCheckException} in the provided {@link MultipleCheckExceptions}
	 * and prints all {@link GeneralCheckException} in the console.
	 * 
	 * @param e
	 */
	protected void reactOnExceptions(final IResource file, final MultipleCheckExceptions e) {
		final Iterator<CheckException> i = e.iterator();
		while (i.hasNext()) {
			final CheckException current = i.next();
			current.accept(new CheckExceptionVisitor() {
				
				@Override
				public void handle(final GeneralCheckException v) {
					MarkerWriter.writeUnknownErrorMarker(file, e.getMessage());
				}
				
				@Override
				public void handle(final MarkableCheckException v) {
					
					MarkerWriter.writeErrorMarker(file, v.getStartPosition().getLineNumber(), v.getStartPosition()
							.getPosition(), v.getEndPos().getPosition(), v.getDescription(), v.getMarker());
				}
			});
		}
	}
	
	protected void reactOnUnknownException(final IResource file, final Exception e) {
		MarkerWriter.writeUnknownErrorMarker(file, e.getMessage());
	}
	
	protected void clear(final IResource file) {
		MarkerWriter.releaseAllParserMarkersOnFile(file);
		ConsoleWriter.clearConsole();
	}
}
